package vip

import (
	"f5"
	"main/constant"
	"main/mt"
	"main/service"
	"math/big"
	"q5"
	"strings"

	"github.com/gin-gonic/gin"
)

type VIPApi struct {
}

func (this *VIPApi) Info(c *gin.Context) {
	accountAddress := c.MustGet("account_address").(string)
	rspObj := struct {
		ErrCode           int32  `json:"errcode"`
		ErrMsg            string `json:"errmsg"`
		ContributionPoint string `json:"contribution_point"`
		Info              struct {
			BindPassport string `json:"bind_passport_address"`
			Email        string `json:"bind_email"`
		} `json:"info"`
	}{}

	{
		var dbErr error
		sql := "SELECT passport_address,lower_case_email FROM t_vip_bind WHERE account_address = ?;"
		params := []string{
			accountAddress,
		}
		f5.GetGoStyleDb().RawQuery(
			constant.BCNFT_DB,
			sql,
			params,
			func(err error, ds *f5.DataSet) {
				dbErr = err
				if err != nil {
					return
				}
				if ds.Next() {
					rspObj.Info.BindPassport = ds.GetByIndex(0)
					rspObj.Info.Email = ds.GetByIndex(1)
				}
			})

		if dbErr != nil {
			f5.RspErr(c, 500, "server internal error")
			return
		}
	}

	c.JSON(200, rspObj)
}

func (this *VIPApi) Bind(c *gin.Context) {
	accountAddress := c.GetString("account_address")
	if accountAddress == "" {
		f5.RspErr(c, 1, "bad request")
		return
	}
	reqObj := struct {
		Passport_jwt string `json:"passport_jwt"`
	}{}

	if err := c.ShouldBindJSON(&reqObj); err != nil {
		f5.RspErr(c, 1, err.Error())
		return
	}

	info := map[string]interface{}{}
	info["account_address"] = ""
	info["email"] = ""
	service.JwtAuth.JwtAuthHandle(reqObj.Passport_jwt, &info, constant.JWT_PASSPORT)
	passport_addr := info["account_address"].(string)
	if passport_addr == "" || accountAddress == passport_addr {
		f5.RspErr(c, 1, "bad request passport")
		return
	}

	email := info["email"].(string)

	bound := false
	{
		var dbErr error
		sql := "SELECT idx FROM t_vip_bind WHERE account_address = ? OR passport_address = ?;"
		params := []string{
			accountAddress,
			passport_addr,
		}
		f5.GetGoStyleDb().RawQuery(
			constant.BCNFT_DB,
			sql,
			params,
			func(err error, ds *f5.DataSet) {
				dbErr = err
				if err != nil {
					return
				}

				if ds.Next() {
					bound = true
				}
			})

		if dbErr != nil {
			f5.RspErr(c, 500, "server internal error")
			return
		}
	}

	if bound {
		f5.RspErr(c, 1, "account or passport bound")
		return
	}

	{
		ts := f5.GetApp().GetRealSeconds()
		var dbErr error
		f5.GetGoStyleDb().Insert(
			constant.BCNFT_DB,
			"t_vip_bind",
			[][]string{
				{"account_address", accountAddress},
				{"passport_address", passport_addr},
				{"email", email},
				{"lower_case_email", strings.ToLower(email)},
				{"createtime", q5.ToString(ts)},
				{"modifytime", q5.ToString(ts)},
			},
			func(err error, lastInsertId int64, rowsAffected int64) {
				if err != nil {
					dbErr = err
					return
				}
			})

		if dbErr != nil {
			f5.RspErr(c, 500, "server internal error")
			return
		}
	}

	f5.RspErr(c, 0, "")
}

func (this *VIPApi) Details(c *gin.Context) {
	accountAddress := strings.ToLower(c.GetString("account_address"))
	if accountAddress == "" {
		f5.RspErr(c, 1, "bad request")
		return
	}

	rspObj := struct {
		ErrCode int32  `json:"errcode"`
		ErrMsg  string `json:"errmsg"`
		Info    struct {
			Level  int32 `json:"level"`
			Cur    int64 `json:"curpoint"`
			Target int64 `json:"target"`
		} `json:"info"`
	}{}

	{
		var dbErr error
		sql := "SELECT * FROM t_vip_user WHERE account_address = ?"
		params := []string{
			accountAddress,
		}
		f5.GetGoStyleDb().RawQuery(
			constant.BCNFT_DB,
			sql,
			params,
			func(err error, ds *f5.DataSet) {
				dbErr = err
				if err != nil {
					return
				}

				if ds.Next() {
					rspObj.Info.Level = q5.SafeToInt32(ds.GetByName("vip_lv"))
					cfg := mt.Table.StakingVip.GetById(q5.SafeToInt64(rspObj.Info.Level))
					if cfg != nil {
						rspObj.Info.Target = cfg.GetThisPoint()
						balancestr := ds.GetByName("escec_balance")
						if balancestr == "" {
							balancestr = "0"
						}
						escec_balance, ok := new(big.Int).SetString(balancestr, 10)
						if !ok {
							f5.GetSysLog().Warning("%s escec_balance err:%s", accountAddress, ds.GetByName("escec_balance"))
							escec_balance = new(big.Int).SetInt64(0)
						}

						stackingstr := ds.GetByName("escec_stacking")
						if stackingstr == "" {
							stackingstr = "0"
						}
						escec_stacking, ok := new(big.Int).SetString(stackingstr, 10)
						if !ok {
							f5.GetSysLog().Warning("%s escec_stacking err:%s", accountAddress, ds.GetByName("escec_stacking"))
							escec_stacking = new(big.Int).SetInt64(0)
						}

						curExec := new(big.Int).Add(escec_balance, escec_stacking)

						convertstr := ds.GetByName("escec_convert")
						if convertstr == "" {
							convertstr = "0"
						}
						escec_convert, ok := new(big.Int).SetString(convertstr, 10)
						if !ok {
							f5.GetSysLog().Warning("%s escec_convert err:%s", accountAddress, ds.GetByName("escec_convert"))
							escec_convert = new(big.Int).SetInt64(0)
						}

						last_convert_time := q5.SafeToInt64(ds.GetByName("last_convert_time"))
						nowsecs := f5.GetApp().GetRealSeconds()
						diff := nowsecs - last_convert_time
						if diff < 0 {
							diff = 0
							f5.GetSysLog().Warning("%s last_convert_time err:%s, now:%d", accountAddress, ds.GetByName("last_convert_time"), nowsecs)
						}
						elapse := new(big.Int).SetInt64(diff)
						depositExec := new(big.Int).Mul(escec_convert, elapse)
						depositExec = depositExec.Div(depositExec, new(big.Int).SetInt64(365*24*3600))
						if depositExec.Cmp(escec_convert) < 0 {
							depositExec = new(big.Int).Sub(escec_convert, depositExec)
						} else {
							depositExec.SetInt64(0)
						}
						curExec = curExec.Add(curExec, depositExec)
						convercurExec := new(big.Int).Div(curExec, new(big.Int).SetInt64(1000*1000*1000*1000*1000*1000)).Int64()
						nextlvcfg := mt.Table.StakingVip.GetById(q5.SafeToInt64(rspObj.Info.Level + 1))
						if nextlvcfg != nil && convercurExec >= cfg.GetRequire() {
							rspObj.Info.Cur = cfg.GetThisPoint() * (convercurExec - cfg.GetRequire()) / (nextlvcfg.GetRequire() - cfg.GetRequire())
						} else if nextlvcfg == nil {
							rspObj.Info.Cur = rspObj.Info.Target
						}
					}
				} else {
					rspObj.Info.Level = 0
					rspObj.Info.Cur = 0
					cfg := mt.Table.StakingVip.GetById(q5.SafeToInt64(0))
					if cfg != nil {
						rspObj.Info.Target = cfg.GetThisPoint()
					}
				}
			})

		if dbErr != nil {
			f5.RspErr(c, 500, "server internal error")
			return
		}
	}

	c.JSON(200, rspObj)
}
